package lc.kra.cell;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Map;

/**
 * Game of Life 
 *
 * Copyright 2013 Kristian Kraljic
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at

 * http://www.apache.org/licenses/LICENSE-2.0

 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * @author <a href=mailto:kristian@kra.lc>Kristian Kraljic</a>
 */
public class AgeingGameOfLife extends GameOfLife {
	private static final long serialVersionUID = 1l;
	protected final static Map<Attribute,String> attributes;
	static {
		attributes = new HashMap<Attribute,String>(GameOfLife.attributes);
		attributes.put(Attribute.NAME,"Aging Game of Life");
		attributes.put(Attribute.DESCRIPTION,GameOfLife.attributes.get(Attribute.DESCRIPTION)+"\n\nVariation in which every cell has a desired lifespan. Instead of dying immediately the cells lifespan is reduced, until it hits zero.");
	}
	
	private static final int LIFESPAN = 10;
	private int[][] ages;
	
	public AgeingGameOfLife() {
		addActionListener(new ActionListener() {
			@Override public void actionPerformed(ActionEvent event) {
				String[] command = event.getActionCommand().split("|");
				if("set".equals(command[0])&&command.length==3)
					ages[Integer.parseInt(command[1])][Integer.parseInt(command[2])] = 0;
			}
		});
	}
	
	@Override protected void paintCell(Graphics graphics,int row,int column) {
		Color color = Color.PINK;
		for(int age=0;age<ages[row][column];age++)
			color = color.darker();
		graphics.setColor(color);
		super.paintCell(graphics,row,column);
	}

	@Override public void extinguish() {
		super.extinguish();
		ages = new int[ROWS][COLUMNS];
	}
	
	@Override public void compute() {
		super.compute();
		this.cells = age(cells);
		repaint(); fireActionPerformed(new ActionEvent(this,2,"compute"));
	}

	private boolean[][] age(boolean[][] cells) {
		for(int row=0;row<ROWS;row++)
			for(int column=0;column<COLUMNS;column++) {
				if(!this.cells[row][column]&&cells[row][column])
					ages[row][column] = 0;
				else if(this.cells[row][column]&&cells[row][column])
					if(ages[row][column]++>LIFESPAN) {
						cells[row][column] = false;
						ages[row][column] = 0;
					}
			}
		return cells;
	}
	
	@Override public Map<Attribute,String> getAttributes() { return attributes; }
	@Override public String toString() { return attributes.get(Attribute.NAME); }
}